##Library Header
# $Id: $
# Copyright  2003-2009 by IXIA
# All Rights Reserved.
#
# Name:
#    ixia_ldp_api.tcl
#
# Purpose:
#    A script development library containing LDP APIs for test automation
#    with the Ixia chassis.
#
# Author:
#
# Usage:
#    package require Ixia
#
# Description:
#    The procedures contained within this library include:
#
#    - emulation_ldp_config
#    - emulation_ldp_route_config
#    - emulation_ldp_control
#    - emulation_ldp_info
#
# Requirements:
#    ixiaapiutils.tcl, a library containing TCL utilities
#    parseddashedargs.tcl, a library containing the parser utilities
#
# Variables:
#
# Keywords:
#
# Category:
#
################################################################################
#                                                                              #
#                                LEGAL  NOTICE:                                #
#                                ==============                                #
# The following code and documentation (hereinafter "the script") is an        #
# example script for demonstration purposes only.                              #
# The script is not a standard commercial product offered by Ixia and have     #
# been developed and is being provided for use only as indicated herein. The   #
# script [and all modifications, enhancements and updates thereto (whether     #
# made by Ixia and/or by the user and/or by a third party)] shall at all times #
# remain the property of Ixia.                                                 #
#                                                                              #
# Ixia does not warrant (i) that the functions contained in the script will    #
# meet the user's requirements or (ii) that the script will be without         #
# omissions or error-free.                                                     #
# THE SCRIPT IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, AND IXIA        #
# DISCLAIMS ALL WARRANTIES, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE,          #
# INCLUDING BUT NOT LIMITED TO ANY WARRANTY OF MERCHANTABILITY AND FITNESS FOR #
# A PARTICULAR PURPOSE OR OF NON-INFRINGEMENT.                                 #
# THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SCRIPT  IS WITH THE #
# USER.                                                                        #
# IN NO EVENT SHALL IXIA BE LIABLE FOR ANY DAMAGES RESULTING FROM OR ARISING   #
# OUT OF THE USE OF, OR THE INABILITY TO USE THE SCRIPT OR ANY PART THEREOF,   #
# INCLUDING BUT NOT LIMITED TO ANY LOST PROFITS, LOST BUSINESS, LOST OR        #
# DAMAGED DATA OR SOFTWARE OR ANY INDIRECT, INCIDENTAL, PUNITIVE OR            #
# CONSEQUENTIAL DAMAGES, EVEN IF IXIA HAS BEEN ADVISED OF THE POSSIBILITY OF   #
# SUCH DAMAGES IN ADVANCE.                                                     #
# Ixia will not be required to provide any software maintenance or support     #
# services of any kind (e.g., any error corrections) in connection with the    #
# script or any part thereof. The user acknowledges that although Ixia may     #
# from time to time and in its sole discretion provide maintenance or support  #
# services for the script, any such services are subject to the warranty and   #
# damages limitations set forth herein and will not obligate Ixia to provide   #
# any additional maintenance or support services.                              #
#                                                                              #
################################################################################

proc ::ixia::emulation_ldp_config { args } {
    variable executeOnTclServer

    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_ldp_config $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable new_ixnetwork_api
    
    ::ixia::utrackerLog $procName $args
    
    set man_args {
        -mode        CHOICES create delete disable enable modify
        -port_handle REGEXP  ^[0-9]+/[0-9]+/[0-9]+$
    }

    set opt_args {
        -handle
        -label_adv                      CHOICES unsolicited on_demand
                                        DEFAULT unsolicited
        -peer_discovery                 CHOICES link targeted targeted_martini
                                        DEFAULT link
        -count                          NUMERIC
                                        DEFAULT 1
        -interface_handle
        -interface_mode                 CHOICES add modify
                                        DEFAULT modify
        -intf_ip_addr                   IPV4
                                        DEFAULT 0.0.0.0
        -intf_prefix_length             RANGE   1-32
                                        DEFAULT 24
        -intf_ip_addr_step              IPV4
                                        DEFAULT 0.0.1.0
        -loopback_ip_addr               IPV4
        -loopback_ip_addr_step          IPV4
                                        DEFAULT 0.0.1.0
        -lsr_id                         IPV4
        -label_space                    RANGE   0-65535
        -lsr_id_step                    IPV4
                                        DEFAULT 0.0.1.0
        -mac_address_init               MAC
        -mac_address_step               MAC
                                        DEFAULT 0000.0000.0001
        -remote_ip_addr                 IPV4
        -remote_ip_addr_step            IPV4
        -hello_interval                 RANGE   0-65535
        -hello_hold_time                RANGE   0-65535
        -keepalive_interval             RANGE   0-65535
        -keepalive_holdtime             RANGE   0-65535
        -discard_self_adv_fecs          CHOICES 0 1
        -vlan                           CHOICES 0 1
        -vlan_id                        RANGE   0-4095
        -vlan_id_mode                   CHOICES fixed increment
                                        DEFAULT increment
        -vlan_id_step                   RANGE   0-4096
                                        DEFAULT 1
        -vlan_user_priority             RANGE   0-7
                                        DEFAULT 0
        -vpi                            RANGE   0-255
                                        DEFAULT 1
        -vci                            RANGE   0-65535
                                        DEFAULT 10
        -vpi_step                       RANGE   0-255
                                        DEFAULT 1
        -vci_step                       RANGE   0-65535
                                        DEFAULT 1
        -atm_encapsulation              CHOICES VccMuxIPV4Routed VccMuxIPV6Routed VccMuxBridgedEthernetFCS VccMuxBridgedEthernetNoFCS LLCRoutedCLIP LLCBridgedEthernetFCS LLCBridgedEthernetNoFCS
        -auth_mode                      CHOICES null md5
                                        DEFAULT null
        -auth_key                       ANY
        -bfd_registration               CHOICES 0 1
                                        DEFAULT 0
        -bfd_registration_mode          CHOICES single_hop multi_hop
                                        DEFAULT multi_hop
        -atm_range_max_vpi              RANGE   0-255
        -atm_range_min_vpi              RANGE   0-255
        -atm_range_max_vci              RANGE   33-65535
        -atm_range_min_vci              RANGE   33-65535
        -atm_vc_dir                     CHOICES bi_dir uni_dir
                                        DEFAULT bi_dir
        -enable_explicit_include_ip_fec CHOICES 0 1
        -enable_l2vpn_vc_fecs           CHOICES 0 1
        -enable_remote_connect          CHOICES 0 1
        -enable_vc_group_matching       CHOICES 0 1
        -gateway_ip_addr                IPV4
        -gateway_ip_addr_step           IPV4 
                                        DEFAULT 0.0.1.0
        -graceful_restart_enable        CHOICES 0 1
        -no_write                       FLAG
        -reconnect_time                 RANGE 0-300000
        -recovery_time                  RANGE 0-300000
        -reset                          FLAG
        -targeted_hello_hold_time       RANGE   0-65535
        -targeted_hello_interval        RANGE   0-65535
        -override_existence_check       CHOICES 0 1 
                                        DEFAULT 0
        -override_tracking              CHOICES 0 1 
                                        DEFAULT 0
    }

    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_ldp_config $args $man_args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: \
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_ldp_route_config { args } {
    variable executeOnTclServer

    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_ldp_route_config $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable new_ixnetwork_api
    variable ldp_handles_array
    keylset returnList status $::SUCCESS
    
    ::ixia::utrackerLog $procName $args
    
    # Arguments
    set man_args {
        -mode                       CHOICES create modify delete
        -handle
    }

    set opt_args {
        -egress_label_mode               CHOICES nextlabel fixed imnull exnull
                                         DEFAULT nextlabel
        -fec_ip_prefix_start             IPV4
                                         DEFAULT 0.0.0.0
        -fec_ip_prefix_length            RANGE 1-32
                                         DEFAULT 24
        -fec_host_addr                   IP
                                         DEFAULT 0.0.0.0
        -fec_host_prefix_length          RANGE 1-32
                                         DEFAULT 24
        -fec_type                        CHOICES ipv4_prefix host_addr vc
                                         DEFAULT ipv4_prefix
        -fec_vc_atm_enable               CHOICES 0 1 
                                         DEFAULT 1
        -fec_vc_atm_max_cells            RANGE 1-65535 
                                         DEFAULT 1
        -fec_vc_cbit                     CHOICES 0 1
                                         DEFAULT 1
        -fec_vc_ce_ip_addr               IP 
                                         DEFAULT 0.0.0.0
        -fec_vc_ce_ip_addr_inner_step    IP 
                                         DEFAULT 0.0.0.1
        -fec_vc_ce_ip_addr_outer_step    IP 
                                         DEFAULT 0.0.0.0
        -fec_vc_cem_option               RANGE 0-65535 
                                         DEFAULT 0
        -fec_vc_cem_option_enable        CHOICES 0 1 
                                         DEFAULT 0
        -fec_vc_cem_payload              RANGE 48-1023 
                                         DEFAULT 48
        -fec_vc_cem_payload_enable       CHOICES 0 1 
                                         DEFAULT 1
        -fec_vc_count                    NUMERIC
                                         DEFAULT 1
        -fec_vc_fec_type                 CHOICES generalized_id_fec_vpls pw_id_fec
                                         DEFAULT generalized_id_fec_vpls
        -fec_vc_group_id                 RANGE 0-2147483647
        -fec_vc_group_count              RANGE 0-4294967295 
                                         DEFAULT 1
        -fec_vc_id_start                 RANGE 0-2147483647
                                         DEFAULT 1
        -fec_vc_id_step                  RANGE 0-2147483647
                                         DEFAULT 1
        -fec_vc_id_count                 RANGE 0-2147483647
                                         DEFAULT 1
        -fec_vc_intf_mtu_enable          CHOICES 0 1
                                         DEFAULT 1
        -fec_vc_intf_mtu                 RANGE 0-65535
                                         DEFAULT 0
        -fec_vc_intf_desc
        -fec_vc_ip_range_addr_count      RANGE 0-4294967295 
                                         DEFAULT 10
        -fec_vc_ip_range_addr_start      IP 
                                         DEFAULT 0.0.0.0
        -fec_vc_ip_range_addr_inner_step IP 
                                         DEFAULT 0.0.0.1
        -fec_vc_ip_range_addr_outer_step IP 
                                         DEFAULT 0.0.1.0
        -fec_vc_ip_range_enable          CHOICES 0 1 
                                         DEFAULT 0
        -fec_vc_ip_range_prefix_len      RANGE 0-32 
                                         DEFAULT 24
        -fec_vc_label_mode               CHOICES fixed_label increment_label
                                         DEFAULT increment_label
        -fec_vc_label_value_start        RANGE 0-1046400 
                                         DEFAULT 16
        -fec_vc_label_value_step         RANGE 0-1046400 
                                         DEFAULT 0
        -fec_vc_mac_range_count          RANGE 0-4294967295 
                                         DEFAULT 1
        -fec_vc_mac_range_enable         CHOICES 0 1 
                                         DEFAULT 0
        -fec_vc_mac_range_first_vlan_id  RANGE 1-4095 
                                         DEFAULT 100
        -fec_vc_mac_range_repeat_mac     CHOICES 0 1 
                                         DEFAULT 1
        -fec_vc_mac_range_same_vlan      CHOICES 0 1 
                                         DEFAULT 1
        -fec_vc_mac_range_start          MAC 
                                         DEFAULT 0000.0000.000
        -fec_vc_mac_range_vlan_enable    CHOICES 0 1 
                                         DEFAULT 1
        -fec_vc_peer_address             IP
                                         DEFAULT 0.0.0.0
        -fec_vc_type                     CHOICES atm_aal5_vcc
                                         CHOICES atm_cell
                                         CHOICES atm_vcc_1_1
                                         CHOICES atm_vcc_n_1
                                         CHOICES atm_vpc_1_1
                                         CHOICES atm_vpc_n_1
                                         CHOICES cem
                                         CHOICES eth
                                         CHOICES eth_vlan
                                         CHOICES eth_vpls
                                         CHOICES fr_dlci
                                         CHOICES hdlc
                                         CHOICES ppp
                                         CHOICES fr_dlci_rfc4619
                                         DEFAULT eth_vlan
        -hop_count_tlv_enable            CHOICES 0 1
                                         DEFAULT 1
        -hop_count_value                 RANGE 1-255
                                         DEFAULT 1
        -label_msg_type                  CHOICES mapping request
                                         DEFAULT mapping
        -label_value_start               RANGE 0-1048575 
                                         DEFAULT 16
        -lsp_handle
        -next_hop_peer_ip                IP
                                         DEFAULT 0.0.0.0
        -num_lsps                        RANGE 1-34048
                                         DEFAULT 1
        -num_routes                      RANGE 1-16777216
                                         DEFAULT 1
        -packing_enable                  CHOICES 0 1
                                         DEFAULT 0
        -provisioning_model              CHOICES bgp_auto_discovery manual_configuration
        -stale_timer_enable              CHOICES 0 1
                                         DEFAULT 1
        -stale_request_time              RANGE 1-65535
                                         DEFAULT 300
        -no_write                        FLAG
    }
    
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_ldp_route_config $args $man_args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: \
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_ldp_control { args } {
    variable executeOnTclServer

    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_ldp_control $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable new_ixnetwork_api
    variable ldp_handles_array
    
    ::ixia::utrackerLog $procName $args
    
    # Arguments
    set man_args {
        -mode        CHOICES restart start stop
    }

    set opt_args {
        -port_handle REGEXP ^[0-9]+/[0-9]+/[0-9]+$
        -handle
    }

	if {[isUNIX] && [info exists ::ixTclSvrHandle]} {
        set retValueClicks [eval "::ixia::SendToIxTclServer $::ixTclSvrHandle {clock clicks}"]
        set retValueSeconds [eval "::ixia::SendToIxTclServer $::ixTclSvrHandle {clock seconds}"]
    } else {
        set retValueClicks [clock clicks]
        set retValueSeconds [clock seconds]
    }
    keylset returnList clicks [format "%u" $retValueClicks]
    keylset returnList seconds [format "%u" $retValueSeconds]

    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_ldp_control $args $man_args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: \
                    [keylget returnList log]"
        }
        keylset returnList clicks [format "%u" $retValueClicks]
        keylset returnList seconds [format "%u" $retValueSeconds]
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_ldp_info { args } {
    variable executeOnTclServer
    
    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_ldp_info $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable new_ixnetwork_api
    variable ldp_handles_array
    
    ::ixia::utrackerLog $procName $args
    
    # Arguments
    set man_args {
        -mode        CHOICES state  stats  clear_stats  settings
                     CHOICES neighbors  lsp_labels
        -handle
    }

    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_ldp_info $args $man_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: \
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}
